import pygame

from pygame.locals import *

from OpenGL.GL import *

from tdgl import mainloop,usualGL,viewgroup,camera,lighting,picking

import gui
import levels
import map
import robot

def standard_ortho_geometry():
    geom = dict(left=-1.0,right=1.0,top=1.0,bottom=-1.0,near=-1.0,far=100.0)
    scrwidth,scrheight = pygame.display.get_surface().get_size()
    ratio = scrwidth * 1.0 / scrheight
    geom['left'] *= ratio
    geom['right'] *= ratio
    return geom

class Master:
    def __init__(self,fullscreen):
        mainloop.open_window((1024,768),fullscreen)
        usualGL.usual_setup()
        pygame.display.set_caption("Vandal Parts - PyWeek6 Entry - Scavengers")
    def run(self):
        quit = False
        while True:
            quit = TitleScreen().run()
            if quit:
                break
            GameState().run()
        print "Thanks for playing."
        pygame.quit()
        raise SystemExit()

class GameState:
    def __init__(self):
        self.prev_mouse = None
        self.commandwin = gui.CommandWin(geom={"scale":0.08,"pos":(-0.95,-0.8,0.0)})
        self.load_new(levels.level_one)
        self.hud = gui.HUD(geom={"scale":0.5,"pos":(0.8,0.8,0.0)})
        self.ogroup = viewgroup.OrthoGroup("ortho",[self.commandwin,self.hud],geom=standard_ortho_geometry())
    def set_camera(self):
        camera.look_from_spherical(self.lat,self.long,self.zoom,1)
    def load_new(self,lvl):
        self.level = lvl()
        self.vgroup = viewgroup.ViewGroup("pv",[self.level.map] + self.level.robots)
        self.robofight = robot.RoboFight(self.level.map,self.level.robots,levels.big_red_button)
        self.vgroup.light_position((0,0,5,1))
        self.vgroup.light_colour((5,5,5,1),1000)
        self.lat = 50
        self.long = -90
        self.zoom = 15
        camera.look_from_spherical(50,-90,20,100)
        camera.look_at((5,5,0),100)
    def draw_opaque(self):
        usualGL.usual_for_opaque()
        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
        self.vgroup.draw("OPAQUE")
        self.ogroup.draw("OPAQUE")
    def draw_transparent(self):
        usualGL.usual_for_transparent()
        self.vgroup.draw("TRANSPARENT")
        self.ogroup.draw("TRANSPARENT")
    def pick(self,x,y):
        picking.start(x,y,1,1,True)
        self.vgroup.draw('PICK')
        objects = picking.end()
        if objects:
            z1,z2,r = objects[0]
            self.commandwin.detach()
            if r:
                tag,data = r
                if tag == "Robot":
                    camera.look_at(data.pos,100)
                    self.ogroup["hud"].target = r
                    if data.team == "ally":
                        self.commandwin.attach(data)
                elif tag == "Button":
                    if self.robofight.state == "Waiting":
                        self.commandwin.detach()
                        self.robofight.change_state("Fighting")
    def run(self):
        self.clock = pygame.time.Clock()
        glLightModelfv(GL_LIGHT_MODEL_AMBIENT,(0.4,0.4,0.4,1.0))
        while True:
            self.draw_opaque()
            self.draw_transparent()
            pygame.display.flip()
            ms = self.clock.tick(60)
            self.vgroup.step(ms)
            self.ogroup.step(ms)
            self.robofight.step(ms)
            if self.robofight.state == "Victory":
                self.load_new(self.level.next)
            if self.robofight.state == "Defeat":
                return
            event = pygame.event.poll()
            self.ogroup.event(event)
            if event.type == QUIT:
                return
            elif event.type == KEYDOWN:
                if event.key == K_RETURN and self.robofight.state == "Waiting":
                    self.commandwin.detach()
                    self.hud.target = None
                    lev = self.level
                    camera.look_at((lev.width/2,lev.length/2,0))
            elif event.type == MOUSEBUTTONDOWN:
                if event.button == 1 and self.robofight.state == "Waiting":
                    x,y = event.pos
                    y = pygame.display.get_surface().get_height() - y
                    self.pick(x,y)
                elif event.button == 3:
                    self.prev_mouse = event.pos
                elif event.button == 4:
                    self.zoom = max(5,self.zoom - 2)
                    self.set_camera()
                elif event.button == 5:
                    self.zoom = min(50,self.zoom + 2)
                    self.set_camera()
            elif event.type == MOUSEMOTION and event.buttons == (0,0,1):
                rel = event.rel
                self.lat = max(0,min(88,self.lat + rel[1]/3))
                self.long = self.long - min(10,max(-10,rel[0]/2))
                self.set_camera()

class TitleScreen(GameState):
    def __init__(self):
        levels.init()
        self.load_new(levels.level_test)
        self.ogroup = viewgroup.OrthoGroup("ortho",[])
        self.robofight.change_state("Title")
    def run(self):
        self.clock = pygame.time.Clock()
        glLightModelfv(GL_LIGHT_MODEL_AMBIENT,(0.4,0.4,0.4,1.0))
        while True:
            self.draw_opaque()
            self.draw_transparent()
            pygame.display.flip()
            ms = self.clock.tick(30)
            self.vgroup.step(ms)
            self.robofight.step(ms)
            if self.robofight.state == "Defeat":
                self.robofight.change_state("Fighting")
            event = pygame.event.poll()
            if event.type == QUIT:
                return "Quit"
            elif event.type == KEYDOWN:
                if event.key == K_RETURN:
                    return
