import pygame

from pygame.locals import *

from OpenGL.GL import *

from tdgl import lighting,objpart,part,texturefont,texture

class Menu(part.ScalePart):
    def __init__(self,options,height = None,*args,**kwd):
        super(Menu,self).__init__("",*args,**kw)
        self.options = options
        self.height = height
        self.width = 0
        self.font = texturefont.TextureFont()
        for o in options:
            w = self.font.width(o[0])
            if w > self.width:
                self.width = w
        if not self.height:
            self.height = len(options)
        self.sel = 0
    def event(self, event):
        if event.type == KEYDOWN:
            if event.key in (K_RETURN,K_SPACE):
                return self.options[self.sel][1]
            elif event.key == K_UP:
                self.sel = max(0,self.sel - 1)
                self.prepare()
            elif event.key == K_DOWN:
                self.sel = min(len(self.options)-1,self.sel + 1)
                self.prepare()
        else:
            return None
    def prepare(self):
        pass


class ActionIcons:
    def __init__(self,fname_dict):
        self.textures = {}
        for name,fname in fname_dict.items():
            self.textures[name] = texture.get_texture(fname)
    def draw(self,name):
        tx = self.textures.get(name)
        if tx:
            tx.bind()
            glBegin(GL_QUADS)
            for x,y,s,t in [
                (0,0,0,0),(1,0,1,0),(1,1,1,1),(0,1,0,1)
                ]:
                glTexCoord2f(s,t)
                glVertex2f(x,y)
            glEnd()

class CommandWin(part.ScalePart):
    _has_transparent = True
    def __init__(self,*args,**kwd):
        super(CommandWin,self).__init__("comwin",*args,**kwd)
        self._visible = False
        self.target = None
        self.moves = 0
        self.attacks = 0
        self.light = lighting.claim_light()
        lighting.light_position(self.light,(1,0.8,-0.1,0)) #directional light
        lighting.light_colour(self.light,(0.4,0.4,0.1,1))
        self.key_actions = {
            K_LEFT:"Left",
            K_RIGHT:"Right",
            K_UP:"Walk",
            K_x:"Attack",
            K_z:"AttackL"}
        self.icons = ActionIcons(
            {"Left":"icon-left.png",
             "Right":"icon-right.png",
             "Walk":"icon-walk.png",
             "Attack":"icon-attackr.png",
             "AttackL":"icon-attackl.png"})

    def __del__(self):
        lighting.release_light(self.light)

    def attach(self,target):
        self.target = target
        self._visible = True

    def detach(self):
        self.target = None
        self._visible = False

    def setup_style(self):
        lighting.light_switch(self.light,True)
        lighting.setup()

    def setdown_style(self):
        lighting.glDisable(GL_LIGHT0 + self.light)
        lighting.light_switch(self.light,False)

    def render(self,mode):
        if self.target is None:
            return
        if mode in ("TRANSPARENT", "OPAQUE"):
            glPushAttrib(GL_LIGHTING)
            glDisable(GL_LIGHTING)
            glPushMatrix()
            glTranslate(-1.0,0,-1.0)
            glDisable(GL_TEXTURE_2D)

            glColor4f(0.0,0.3,0.0,0.5)
            glBegin(GL_QUADS)
            glVertex3f(0.0,0.8,-0.2)
            glVertex3f(0.0,-0.8,-0.2)
            glVertex3f(20.0,-0.8,-0.2)
            glVertex3f(20.0,0.8,-0.2)
            glEnd()

            glColor4f(0.8,1.0,0.8,0.95)
            glBegin(GL_LINE_LOOP)
            glVertex3f(0.0,0.8,0.0)
            glVertex3f(0.0,-0.8,0.0)
            glVertex3f(20.0,-0.8,0.0)
            glVertex3f(20.0,0.8,0.0)
            glEnd()
            glPopMatrix()
            glPushMatrix()
            glEnable(GL_TEXTURE_2D)
            glTranslate(-0.2,-0.4,0)
            glColor4f(1,1,1,1)
            for action in self.target.program:
                self.icons.draw(action)
                glTranslate(1.3,0,0)
            glPopMatrix()
            glPopAttrib()

    def add_action(self,action):
        self.target.program.append(action)

    def event(self,evt):
        r = self.target
        if evt.type != KEYDOWN or r is None:
            return
        action = self.key_actions.get(evt.key)
        if action and len(r.program) < r.moves:
            if action.startswith("Attack"):
                attacks = len([a for a in r.program
                               if a.startswith("Attack")])
                if attacks >= r.attacks:
                    return
            self.add_action(action)
        elif evt.key == K_BACKSPACE:
            self.target.program[-1:] = []

class HUD(part.ScalePart):
    _has_transparent = True
    def __init__(self,*args,**kwd):
        super(HUD,self).__init__("hud",*args,**kwd)
        self._visible = False
        self.colours = {
            "blue":[(0.0,0.7,1.0,0.5),(0.0,0.0,1.0,0.7)],
            "yellow":[(0.7,0.7,0.0,0.5),(1.0,1.0,0.0,0.7)]
            }
        self.colour = None
        self.target = None
        self.font = texturefont.TextureFont()
        self.victory = False
        self.defeat = False
    def step(self,ms):
        if self.target:
            self._visible = True
            if self.target[1].team == "ally":
                self.colour = "blue"
            else:
                self.colour = "yellow"
        else:
            self._visible = False
    def setup_style(self):
        glDisable(GL_TEXTURE_2D)
    def render(self,mode):
        if self.target == None:
            return
        if mode == "TRANSPARENT":
            glPushAttrib(GL_LIGHTING)
            glDisable(GL_LIGHTING)
            glPushMatrix()

            glBegin(GL_QUADS)
            glColor4f(*self.colours[self.colour][0])
            glVertex3f(0.0,-1.0,0.01)
            glVertex3f(1.0,-1.0,0.01)
            glColor4f(*self.colours[self.colour][1])
            glVertex3f(1.0,0.0,0.01)
            glVertex3f(0.0,0.0,0.01)
            glEnd()
            
            glTranslate(0.0,-1.52,0.0)
            
            glBegin(GL_QUADS)
            glColor4f(*self.colours[self.colour][0])
            glVertex3f(0.0,-1.0,0.01)
            glVertex3f(1.0,-1.0,0.01)
            glColor4f(*self.colours[self.colour][1])
            glVertex3f(1.0,0.5,0.01)
            glVertex3f(0.0,0.5,0.01)
            glEnd()

            glPopMatrix()
            glPopAttrib()

        if mode == "OPAQUE":
            glPushAttrib(GL_LIGHTING)
            glDisable(GL_LIGHTING)
            glPushMatrix()

            glBegin(GL_LINE_LOOP)
            glColor4f(0.5,0.5,0.5,1.0)
            glVertex3f(0.0,-1.0,0.02)
            glVertex3f(1.0,-1.0,0.02)
            glColor4f(0.8,0.8,0.8,1.0)
            glVertex3f(1.0,0.0,0.02)
            glVertex3f(0.0,0.0,0.02)
            glEnd()

            glBegin(GL_QUADS)
            glColor4f(0.0,0.0,0.0,1.0)
            glVertex3f(0.05,-0.95,0.02)
            glVertex3f(0.95,-0.95,0.02)
            glVertex3f(0.95,-0.85,0.02)
            glVertex3f(0.05,-0.85,0.02)
            glEnd()

            x = (0.9/self.target[1].hp)*self.target[1].health

            glBegin(GL_QUADS)
            glColor4f(0.0,0.8,0.0,1.0)
            glVertex3f(0.05+x,-0.85,0.02)
            glVertex3f(0.05,-0.85,0.02)
            glColor4f(0.0,0.4,0.0,1.0)
            glVertex3f(0.05,-0.95,0.02)
            glVertex3f(0.05+x,-0.95,0.02)
            glEnd()
            
            glBegin(GL_LINE_LOOP)
            glColor4f(0.8,0.8,0.8,1.0)
            glVertex3f(0.05,-0.95,0.04)
            glVertex3f(0.95,-0.95,0.04)
            glVertex3f(0.95,-0.85,0.04)
            glVertex3f(0.05,-0.85,0.04)
            glEnd()

            glTranslate(0.0,-1.52,0.0)
            
            glBegin(GL_LINE_LOOP)
            glColor4f(0.5,0.5,0.5,1.0)
            glVertex3f(0.0,-1.0,0.02)
            glVertex3f(1.0,-1.0,0.02)
            glColor4f(0.8,0.8,0.8,1.0)
            glVertex3f(1.0,0.5,0.02)
            glVertex3f(0.0,0.5,0.02)
            glEnd()

            glPopMatrix()

            glPushMatrix()
            glColor4f(1.0,1.0,1.0,1.0)
            glScalef(0.1,0.1,1.0)
            glTranslate(0.0,0.0,0.03)
            self.font.render(self.target[1].name)
            glTranslate(0.4,-8.0,0.0)
            self.font.render("HP: %d/%d"% (self.target[1].health,self.target[1].hp))
            glTranslate(0.0,-5.0,0.0)
            self.font.render("Power: %d"% self.target[1].power)
            glTranslate(0.0,-1.0,0.0)
            self.font.render("Speed: %d"% self.target[1].speed)
            glTranslate(0.0,-2.0,0.0)
            self.font.render("Moves: %d"% self.target[1].moves)
            glTranslate(0.0,-1.0,0.0)
            self.font.render("Attacks: %d"% self.target[1].attacks)
            glTranslate(0.0,-2.0,0.0)
            self.font.render("Jump: %d"% self.target[1].jump)
            glTranslate(0.0,-1.0,0.0)
            self.font.render("Reach: %d"% self.target[1].reach)
            glTranslate(0.0,-1.0,0.0)
            self.font.render("AI: %d"% self.target[1].clever)
            glDisable(GL_TEXTURE_2D)
            glPopMatrix()

            glPopAttrib()
