import pygame

import blocks,resource

from robot import * 

mapping = {}
big_red_button = None

def init():
    Block = blocks.Block
    Obstacle = blocks.Obstacle
    global big_red_button
    big_red_button = blocks.Button(1.0,"button.obj")

    mapping["@"] = big_red_button

    mapping["h"] = Block(0.5,"block-h5-hazard.obj")
    mapping["b"] = Block(0.5,"block-h5-mesh.obj")
    mapping["f"] = Block(0.1,"block-h0-grate.obj")
    mapping["Z"] = Block(0.5,"block-h10-stairs.obj")
    mapping["S"] = Block(0.5,"block-h10-stairs.obj",angle=180)
    mapping["U"] = Block(0.5,"block-h10-stairs.obj",angle=90)
    mapping["D"] = Block(0.5,"block-h10-stairs.obj",angle=-90)
    mapping["z"] = Block(0.25,"block-h5-stairs.obj")
    mapping["s"] = Block(0.25,"block-h5-stairs.obj",angle=180)
    mapping["u"] = Block(0.25,"block-h5-stairs.obj",angle=90)
    mapping["d"] = Block(0.25,"block-h5-stairs.obj",angle=-90)

    mapping["<"] = Block(-0.5,"block-oil-3s0c.obj")
    mapping["^"] = Block(-0.5,"block-oil-3s0c.obj",angle=-90)
    mapping["v"] = Block(-0.5,"block-oil-3s0c.obj",angle=90)
    mapping[">"] = Block(-0.5,"block-oil-3s0c.obj",angle=180)

    mapping["#"] = Block(-0.5,"block-oil-0s4c.obj")

    mapping["'"] = Block(-0.5,"block-oil-0s1c.obj")
    mapping["`"] = Block(-0.5,"block-oil-0s1c.obj")
    mapping[","] = Block(-0.5,"block-oil-0s1c.obj")
    mapping["."] = Block(-0.5,"block-oil-0s1c.obj")

    mapping["o"] = Block(-0.5,"block-oil-0s0c.obj")

    mapping["~"] = Block(-0.5,"block-oil-4s0c.obj")

    mapping["F"] = Block(-0.5,"block-oil-2s0c.obj")
    mapping["?"] = Block(-0.5,"block-oil-2s0c.obj",angle=-90)
    mapping["L"] = Block(-0.5,"block-oil-2s0c.obj",angle=90)
    mapping["J"] = Block(-0.5,"block-oil-2s0c.obj",angle=180)

    mapping["/"] = Block(-0.5,"block-oil-2s1c.obj",angle=90)
    mapping["\\"] = Block(-0.5,"block-oil-2s1c.obj")
    mapping["["] = Block(-0.5,"block-oil-2s1c.obj",angle=180)
    mapping["]"] = Block(-0.5,"block-oil-2s1c.obj",angle=-90)

    mapping["%"] = Block(-0.5,"block-oil-0s2c.obj",angle=180)
    mapping['"'] = Block(-0.5,"block-oil-0s2c.obj",angle=180)
    mapping[":"] = Block(-0.5,"block-oil-0s2c.obj",angle=-90)
    mapping[";"] = Block(-0.5,"block-oil-0s2c.obj",angle=180)

    mapping["H"] = Block(-0.5,"block-oil-iso2s0c.obj",angle=90)
    mapping["="] = Block(-0.5,"block-oil-iso2s0c.obj")

    mapping["-"] = Block(-0.5,"block-oil-1s0c.obj",angle=180)
    mapping["_"] = Block(-0.5,"block-oil-1s0c.obj",angle=180)
    mapping["|"] = Block(-0.5,"block-oil-1s0c.obj",angle=90)
    mapping["I"] = Block(-0.5,"block-oil-1s0c.obj",angle=-90)

    mapping["V"] = Block(0.75,"letter-v.obj",angle=-90)
    mapping["a"] = Block(0.75,"letter-a.obj",angle=-90)
    mapping["n"] = Block(0.75,"letter-n.obj",angle=-90)
    mapping["9"] = Block(0.75,"letter-d.obj",angle=-90)
    mapping["l"] = Block(0.75,"letter-l.obj",angle=-90)
    mapping["p"] = Block(0.75,"letter-p.obj",angle=-90)
    mapping["r"] = Block(0.75,"letter-r.obj",angle=-90)
    mapping["t"] = Block(0.75,"letter-t.obj",angle=-90)
    mapping["5"] = Block(0.75,"letter-s.obj",angle=-90)

class Level:
    def __init__(self,length,width,stacks,robots,next_level):
        self.length = length
        self.width = width
        self.robots = list(robots)
        self.map = map = blocks.StackMap(length,width)
        self.next = next_level
        for x,y,s in stacks:
            map.add(x,y,s,mapping)
            all_tiles = [(x,y) for x in range(width) for y in range(length)]
        for x,y in all_tiles:
            h = map.floor_height(x,y)
            if h == None or h > 0:
                map.add_under(x,y,"f",mapping)
        for r in robots:
            x,y,z = r.pos
            h = map.floor_height(x,y)
            if h is not None:
                r.pos = (x,y,h)
            else:
                self.robots.remove(r)


def make_robot(name,pieces,x,y,team):
    rpieces = []
    for p,c in pieces.items():
        if p == "arml":
            rpieces.append(c(p,flip=True))
        else:
            rpieces.append(c(p))
    robo = Robot(name,rpieces,team)
    robo.pos = x,y,0
    return robo

def level_one():
    stacks = [
    (1,1,"bh"),
    (1,2,"b"),
    (1,3,"b"),
    (1,7,"b"),
    (1,8,"b"),
    (1,9,"bh"),
    (2,1,"b"),
    (2,9,"b"),
    (3,1,"b"),
    (3,3,"~"),
    (3,5,"S"),
    (3,7,"~"),
    (3,9,"b"),
    (4,4,"h"),
    (4,5,"bbh"),
    (4,6,"h"),
    (5,3,"D"),
    (5,4,"bbh"),
    (5,5,"bbbh@"),
    (5,6,"bbh"),
    (5,7,"U"),
    (6,4,"h"),
    (6,5,"bbh"),
    (6,6,"h"),
    (7,1,"b"),
    (7,3,"~"),
    (7,5,"Z"),
    (7,7,"~"),
    (7,9,"b"),
    (8,1,"b"),
    (8,9,"b"),
    (9,1,"bh"),
    (9,2,"b"),
    (9,3,"b"),
    (9,7,"b"),
    (9,8,"b"),
    (9,9,"bh"),
    ]
    
    robots=[]
    def add_robot(*args):
        robots.append(make_robot(*args))
    add_robot("Jeffrey",{"head":ScopeHead,
                         "body":SpringBody,
                         "legs":StupidLegs,
                         "armr":ChainsawArm,
                         "arml":VandalPart},5,2,"ally")
    
    add_robot("Marvin",{"head":CyclopseHead,
                        "body":AndroidBody,
                        "legs":AndroidLegs,
                        "arml":AxeArm},2,2,"enemy")
    add_robot("Robbie",{"head":AndroidHead,
                        "body":GlassBody,
                        "legs":HoverLegs,
                        "arml":AxeArm},8,8,"enemy")
    add_robot("Gregory",{"head":CyclopseHead,
                         "body":GlassBody,
                         "legs":AndroidLegs,
                         "arml":AxeArm},2,8,"enemy")
    add_robot("MP3PO",{"head":CyclopseHead,
                       "body":AndroidBody,
                       "legs":StupidLegs,
                       "arml":AxeArm},8,2,"enemy")
    next_level = level_two
    return Level(11,11,stacks,robots,next_level)

def level_two():
    stacks = [
    (0,10,"h"),
    (0,2,"<"),
    (0,4,"h"),
    (0,5,"bh"),
    (0,6,"bhD"),
    (0,7,"bbbh"),
    (0,8,"bhU"),
    (0,9,"bh"),
    (1,10,"h"),
    (1,2,"="),
    (1,4,"h"),
    (1,5,"bh"),
    (1,6,"bhD"),
    (1,7,"bbbh"),
    (1,8,"bhU"),
    (1,9,"bh"),
    (10,10,"h"),
    (10,2,"="),
    (10,4,"h"),
    (10,5,"bh"),
    (10,6,"bhD"),
    (10,7,"bbbh"),
    (10,8,"bhU"),
    (10,9,"bh"),
    (11,10,"h"),
    (11,2,">"),
    (11,4,"h"),
    (11,5,"bh"),
    (11,6,"bhD"),
    (11,7,"bbbh"),
    (11,8,"bhU"),
    (11,9,"bh"),
    (2,10,"h"),
    (2,2,"="),
    (2,4,"h"),
    (2,5,"bh"),
    (2,6,"bhD"),
    (2,7,"bbbh"),
    (2,8,"bhU"),
    (2,9,"bh"),
    (3,10,"h"),
    (3,2,"="),
    (3,4,"h"),
    (3,5,"bh"),
    (3,6,"bhD"),
    (3,7,"bbbh"),
    (3,8,"bhU"),
    (3,9,"bh"),
    (4,1,"bh"),
    (4,10,"h"),
    (4,2,">"),
    (4,4,"h"),
    (4,5,"bh"),
    (4,6,"bhD"),
    (4,7,"bbbh"),
    (4,8,"bhU"),
    (4,9,"bh"),
    (5,1,"h"),
    (5,10,"h"),
    (5,4,"h"),
    (5,5,"bh"),
    (5,6,"bhD"),
    (5,7,"bbbh@"),
    (5,8,"bhU"),
    (5,9,"bh"),
    (6,1,"h"),
    (6,10,"h"),
    (6,4,"h"),
    (6,5,"bh"),
    (6,6,"bhD"),
    (6,7,"bbbh"),
    (6,8,"bhU"),
    (6,9,"bh"),
    (7,1,"bh"),
    (7,10,"h"),
    (7,2,"<"),
    (7,4,"h"),
    (7,5,"bh"),
    (7,6,"bhD"),
    (7,7,"bbbh"),
    (7,8,"bhU"),
    (7,9,"bh"),
    (8,10,"h"),
    (8,2,"="),
    (8,4,"h"),
    (8,5,"bh"),
    (8,6,"bhD"),
    (8,7,"bbbh"),
    (8,8,"bhU"),
    (8,9,"bh"),
    (9,10,"h"),
    (9,2,"="),
    (9,4,"h"),
    (9,5,"bh"),
    (9,6,"bhD"),
    (9,7,"bbbh"),
    (9,8,"bhU"),
    (9,9,"bh"),

    ]
    
    robots=[]
    def add_robot(*args):
        robots.append(make_robot(*args))
    add_robot("Jeffrey",{"head":ScopeHead,
                         "body":SpringBody,
                         "legs":StupidLegs,
                         "armr":ChainsawArm,
                         "arml":LanceArm},5,0,"ally")
    add_robot("Gregory",{"head":CyclopseHead,
                         "body":GlassBody,
                         "legs":AndroidLegs,
                         "arml":AxeArm},6,0,"ally")

    add_robot("Zeke",{"head":CyclopseHead,
                      "body":AndroidBody,
                      "legs":StupidLegs,
                      "arml":AxeArm},3,11,"enemy")
    add_robot("Alice",{"head":AndroidHead,
                       "body":AndroidBody,
                       "legs":StupidLegs,
                       "arml":LanceArm},4,11,"enemy")
    add_robot("Kryton",{"head":AndroidHead,
                        "body":AndroidBody,
                        "legs":AndroidLegs,
                        "arml":LanceArm,
                        "armr":LanceArm},5,11,"enemy")
    add_robot("Sunny",{"head":CyclopseHead,
                       "body":GlassBody,
                       "legs":StupidLegs,
                       "armr":LanceArm},6,11,"enemy")
    add_robot("HAL",{"head":CyclopseHead,
                     "body":AndroidBody,
                     "legs":AndroidLegs,
                     "armr":TenderiserArm},7,11,"enemy")

    next_level = level_three

    return Level(12,12,stacks,robots,next_level)

def level_three():
    stacks = [
    (2,7,"h"),
    (2,8,"bhd"),
    (2,9,"bbh"),
    (3,2,"<"),
    (3,4,"<"),
    (3,9,"bbh"),
    (4,1,"v"),
    (4,2,"#"),
    (4,3,"H"),
    (4,4,"#"),
    (4,5,"^"),
    (4,9,"bbhS"),
    (5,2,"="),
    (5,3,"@"),
    (5,4,"="),
    (5,9,"bbbbh"),
    (6,1,"v"),
    (6,2,"#"),
    (6,3,"H"),
    (6,4,"#"),
    (6,5,"^"),
    (6,9,"bbhZ"),
    (7,2,">"),
    (7,4,">"),
    (7,9,"bbh"),
    (8,7,"h"),
    (8,8,"bhd"),
    (8,9,"bbh"),
        ]
    robots=[]
    def add_robot(*args):
        robots.append(make_robot(*args))
    add_robot("Jeffrey",{"head":ScopeHead,
                         "body":SpringBody,
                         "legs":StupidLegs,
                         "armr":ChainsawArm,
                         "arml":TenderiserArm},4,0,"ally")
    add_robot("Gregory",{"head":CyclopseHead,
                         "body":GlassBody,
                         "legs":HoverLegs,
                         "armr":LanceArm,
                         "arml":AxeArm},6,0,"ally")
    add_robot("Kryton",{"head":AndroidHead,
                         "body":AndroidBody,
                         "legs":AndroidLegs,
                         "armr":LanceArm,
                         "arml":LanceArm},5,0,"ally")
    
    add_robot("Zeta",{"head":MilitaryHead,
                        "body":GlassBody,
                        "legs":HoverLegs,
                        "arml":TenderiserArm},1,9,"enemy")
    add_robot("Tachikoma",{"head":AndroidHead,
                        "body":AndroidBody,
                        "legs":MilitaryLegs,
                        "arml":LanceArm},3,3,"enemy")
    add_robot("Steeve",{"head":MilitaryHead,
                        "body":MilitaryBody,
                        "legs":MilitaryLegs,
                        "armr":TenderiserArm,
                        "arml":HellstarArm},5,7,"enemy")
    add_robot("Ming",{"head":CyclopseHead,
                       "body":AndroidBody,
                       "legs":MilitaryLegs,
                       "arml":LanceArm},7,3,"enemy")
    add_robot("Beta",{"head":MilitaryHead,
                       "body":GlassBody,
                       "legs":HoverLegs,
                       "arml":HellstarArm},9,9,"enemy")
    next_level = level_four
    return Level(11,11,stacks,robots,next_level)

def level_four():
    stacks = [
        (0,6,"D"),
        (0,7,"U"),
        (10,0,"S"),
        (10,11,"bh"),
        (10,13,"s"),
        (10,3,"bh"),
        (11,0,"Z"),
        (11,10,"b"),
        (11,11,"bbbbbbbh"),
        (11,13,"z"),
        (11,3,"bbbbbbbh"),
        (11,4,"b"),
        (11,5,"b"),
        (11,6,"b"),
        (11,7,"b"),
        (11,8,"b"),
        (11,9,"b"),
        (13,6,"D"),
        (13,7,"U"),
        (2,0,"S"),
        (2,10,"b"),
        (2,11,"bbbbbbbh"),
        (2,13,"s"),
        (2,3,"bbbbbbbh"),
        (2,4,"b"),
        (2,5,"b"),
        (2,6,"b"),
        (2,7,"b"),
        (2,8,"b"),
        (2,9,"b"),
        (3,0,"Z"),
        (3,11,"bh"),
        (3,13,"z"),
        (3,3,"bh"),
        (4,0,"v"),
        (4,1,"/"),
        (4,11,"bbh"),
        (4,3,"bbh"),
        (4,7,"~"),
        (4,9,"~"),
        (5,1,"="),
        (5,11,"bh"),
        (5,3,"bh"),
        (6,1,"="),
        (6,10,"h"),
        (6,11,"h"),
        (6,3,"h"),
        (6,4,"h"),
        (6,5,"h"),
        (6,6,"bbbh@"),
        (6,7,"hs"),
        (6,8,"h"),
        (6,9,"h"),
        (7,1,"="),
        (7,10,"h"),
        (7,11,"h"),
        (7,3,"h"),
        (7,4,"h"),
        (7,5,"h"),
        (7,6,"bbhz"),
        (7,7,"bhu"),
        (7,8,"h"),
        (7,9,"h"),
        (8,1,"="),
        (8,11,"bh"),
        (8,3,"bh"),
        (9,0,"v"),
        (9,1,"\\"),
        (9,11,"bbh"),
        (9,3,"bbh"),
        (9,7,"~"),
        (9,9,"~"),

        ]
    robots=[]
    def add_robot(*args):
        robots.append(make_robot(*args))
    add_robot("Jeffrey",{"head":ScopePlusHead,
                         "body":SpringBody,
                         "legs":StupidLegs,
                         "armr":ChainsawArm,
                         "arml":TenderiserArm},6,0,"ally")
    add_robot("Gregory",{"head":CyclopseHead,
                         "body":GlassBody,
                         "legs":HoverLegs,
                         "armr":LanceArm,
                         "arml":AxeArm},8,0,"ally")
    add_robot("Kryton",{"head":AndroidHead,
                         "body":AndroidBody,
                         "legs":AndroidLegs,
                         "armr":LanceArm,
                         "arml":LanceArm},7,0,"ally")
    add_robot("Steeve",{"head":MilitaryHead,
                        "body":MilitaryBody,
                        "legs":MilitaryLegs,
                        "armr":TenderiserArm,
                        "arml":HellstarArm},5,0,"ally")
    
    add_robot("Miyamoto",{"head":BushidoHead,
                        "body":BushidoBody,
                        "legs":BushidoLegs,
                        "armr":KatanaArm,
                        "arml":KatanaArm},10,10,"enemy")
    add_robot("Gennosuke",{"head":BushidoHead,
                        "body":BushidoBody,
                        "legs":BushidoLegs,
                        "arml":LanceArm},3,10,"enemy")
    add_robot("Kotetsu",{"head":BushidoHead,
                        "body":GlassBody,
                        "legs":AndroidLegs,
                        "armr":AxeArm,
                        "arml":HellstarArm},4,5,"enemy")
    add_robot("Watanabe",{"head":BushidoHead,
                       "body":BushidoBody,
                       "legs":BushidoLegs,
                       "arml":AxeArm},5,10,"enemy")
    add_robot("Jiro",{"head":BushidoHead,
                       "body":BushidoBody,
                       "legs":BushidoLegs,
                       "arml":AxeArm},8,10,"enemy")
    add_robot("Kaonashi",{"body":BushidoBody,
                       "legs":NofaceLegs,
                       "armr":KatanaArm,
                       "arml":KatanaArm},9,5,"enemy")
    next_level = level_five
    return Level(14,14,stacks,robots,next_level)

def level_five():
    stacks = [
    (0,1,"bbh"),
    (0,2,"bbh"),
    (0,3,"bbh"),
    (0,4,"bbh"),
    (0,5,"bbh"),
    (0,6,"bbh"),
    (0,7,"<"),
    (1,6,"bbh"),
    (1,7,"="),
    (10,11,"z"),
    (10,6,"bbh"),
    (10,7,"="),
    (11,6,"bbh"),
    (11,7,"="),
    (12,1,"bbh"),
    (12,2,"bbh"),
    (12,3,"bbh"),
    (12,4,"bbh"),
    (12,5,"bbh"),
    (12,6,"bbh"),
    (12,7,">"),
    (2,11,"s"),
    (2,6,"bbh"),
    (2,7,"="),
    (3,11,"bh"),
    (3,6,"bh"),
    (3,7,"="),
    (3,8,"L"),
    (3,9,"F"),
    (4,11,"bh"),
    (4,6,"bbh"),
    (4,7,"="),
    (4,8,"J"),
    (4,9,"?"),
    (5,11,"bh"),
    (5,6,"bbh@"),
    (5,7,"="),
    (6,10,"bh"),
    (6,11,"bbh"),
    (6,12,"bb"),
    (6,7,"="),
    (6,8,"D"),
    (6,9,"bh"),
    (7,11,"bh"),
    (7,6,"bbh@"),
    (7,7,"="),
    (8,11,"bh"),
    (8,6,"bbh"),
    (8,7,"="),
    (8,8,"L"),
    (8,9,"F"),
    (9,11,"bh"),
    (9,6,"bh"),
    (9,7,"="),
    (9,8,"J"),
    (9,9,"?"),


        ]
    robots=[]
    def add_robot(*args):
        robots.append(make_robot(*args))
    add_robot("Jeffrey -Vandalgear",{"head":VandalHead,
                         "body":VandalBody,
                         "legs":VandalLegs,
                         "armr":VandalPart,
                         "arml":ChainsawArm},6,2,"ally")
    add_robot("Gregory",{"head":CyclopseHead,
                         "body":GlassBody,
                         "legs":HoverLegs,
                         "armr":LanceArm,
                         "arml":AxeArm},6,3,"ally")
    add_robot("Kryton",{"head":AndroidHead,
                         "body":AndroidBody,
                         "legs":AndroidLegs,
                         "armr":LanceArm,
                         "arml":LanceArm},6,1,"ally")
    add_robot("Steeve",{"head":MilitaryHead,
                        "body":MilitaryBody,
                        "legs":MilitaryLegs,
                        "armr":TenderiserArm,
                        "arml":HellstarArm},5,2,"ally")
    add_robot("Miyamoto",{"head":BushidoHead,
                        "body":BushidoBody,
                        "legs":BushidoLegs,
                        "armr":KatanaArm,
                        "arml":KatanaArm},7,2,"ally")

    add_robot("X",{"head":MilitaryHead,
                        "body":MilitaryBody,
                        "legs":HoverLegs,
                        "armr":MaulArm,
                        "arml":HellstarArm},5,10,"enemy")
    add_robot("Y",{"head":BushidoHead,
                        "body":BushidoBody,
                        "legs":HoverLegs,
                        "armr":KatanaArm,
                        "arml":MaulArm},7,10,"enemy")
    add_robot("Z",{"head":ScopeHead,
                        "body":SpringBody,
                        "legs":HoverLegs,
                        "armr":KatanaArm,
                        "arml":ChainsawArm},6,6,"enemy")
    add_robot("OoX M. Lord",{"head":GorillaHead,
                             "body":GorillaBody,
                             "legs":GorillaLegs,
                             "armr":DrainArm,
                             "arml":GorillaArm},6,11,"enemy")

    next_level = level_five
    return Level(13,13,stacks,robots,next_level)

def level_test():
    stacks = [
(1,1,"L"),
    (1,2,"F"),
    (1,5,"<"),
    (1,8,"bV"),
    (2,1,"_"),
    (2,2,'"'),
    (2,3,"H"),
    (2,4,"H"),
    (2,5,"\\"),
    (2,8,"ba"),
    (3,1,"J"),
    (3,2,"?"),
    (3,8,"bn"),
    (4,7,"p"),
    (4,8,"b9"),
    (5,7,"a"),
    (5,8,"ba"),
    (6,7,"r"),
    (6,8,"bl"),
    (7,3,"S"),
    (7,7,"t"),
    (8,3,"bh"),
    (8,7,"5"),
    (9,2,"D"),
    (9,3,"bh"),
    (9,4,"bh"),
    (9,5,"U"),

              ]
    robots = []
    def add_robot(*args):
        robots.append(make_robot(*args))

    add_robot("OoX M. Lord",{"head":GorillaHead,
                             "body":GorillaBody,
                             "legs":GorillaLegs,
                             "armr":DrainArm,
                             "arml":GorillaArm},5,5,"enemy")

    next_level = level_one
    return Level(10,10,stacks,robots,next_level)
